@@ -102,9 +102,6 @@ namespace
     return ret;
   }
 
-  inline const char * lockStatusTag( const ui::Selectable & sel_r )
-  { return ::lockStatusTag( sel_r.installedEmpty() ? "" : "i", sel_r.locked(), sel_r.identIsAutoInstalled() ); }
-
 } // namespace
 ///////////////////////////////////////////////////////////////////
 
@@ -453,18 +450,18 @@ void printPatternInfo(Zypper & zypper, const ui::Selectable & s , const PrintInf
       for ( ui::Selectable::Ptr sel : collect.req.selectable() )
       {
 	const ui::Selectable & s = *sel;
-	t << ( TableRow() << lockStatusTag( s ) << s.name() << s.kind().asString() << _("Required") );
+	t << ( TableRow() << computeStatusIndicator( s ) << s.name() << s.kind().asString() << _("Required") );
       }
       for ( ui::Selectable::Ptr sel : collect.rec.selectable() )
       {
 	const ui::Selectable & s = *sel;
-	t << ( TableRow() << lockStatusTag( s ) << s.name() << s.kind().asString() << _("Recommended") );
+	t << ( TableRow() << computeStatusIndicator( s ) << s.name() << s.kind().asString() << _("Recommended") );
       }
       if ( showSuggests )
 	for ( ui::Selectable::Ptr sel : collect.sug.selectable() )
 	{
 	  const ui::Selectable & s = *sel;
-	  t << ( TableRow() << lockStatusTag( s ) << s.name() << s.kind().asString() << _("Suggested") );
+	  t << ( TableRow() << computeStatusIndicator( s ) << s.name() << s.kind().asString() << _("Suggested") );
 	}
 
       std::map<std::string, unsigned> depPrio({{_("Required"),0}, {_("Recommended"),1}, {_("Suggested"),2}});
@@ -659,7 +656,8 @@ namespace
     return ret;
   }
 
-  // TODO Use a common status tag computation here, in search.cc, ...
+  // Slightly different from computeStatusIndicator: 'i' if exactly the
+  // source packages version is installed.
   const char * BinPkgStatus( const BinPkg & binPkg_r )
   {
     ui::Selectable::Ptr sel( ui::Selectable::get( binPkg_r.first ) );
@@ -669,6 +667,7 @@ namespace
       if ( pi.edition() == binPkg_r.second )
 	return "i";
     return "v";
+    return computeStatusIndicator( *sel , binPkg_r.second );
   }
 
 } // namespace
