@@ -9,6 +9,7 @@
 #include "exceptions.h"
 #include "utils/messages.h"
 #include "Zypper.h"
+#include <zypp/base/Regex.h>
 
 #include <getopt.h>
 #include <unordered_map>
@@ -62,7 +63,15 @@ namespace {
 
     //we do not use the flag and val types, instead we use optind to figure out what happend
     using OptType = struct option;
-    opts.push_back( OptType{opt.name.c_str(), has_arg, 0 ,0} );
+
+    opts.push_back( OptType{
+      opt.name.c_str(),
+      has_arg,
+      0 ,
+      // set the "val" member to some unique number to allow a correct detection of a ambigous switch name (bsc#1165573)
+      // make sure the number is negative so we can differ between short and long options in case of error
+      - 100 - static_cast<int>(opts.size())
+    });
   }
 }
 
@@ -163,6 +172,16 @@ Value & Value::after( PostWriteHook &&postWriteHook )
 }
 
 
+bool &onlyWarnOnAbbrevSwitches()
+{
+#if LEGACY_ENABLE_LONGOPT_ABBREV
+  static bool enable = true;
+#else
+  static bool enable = false;
+#endif
+  return enable;
+}
+
 int parseCLI( const int argc, char * const *argv, const std::vector<CommandGroup> &options )
 {
   // the short options string as used int getopt
@@ -300,10 +319,15 @@ int parseCLI( const int argc, char * const *argv, const std::vector<CommandGroup
   //the key of the map is the index of options in allOpts
   std::map<int, std::vector< boost::optional<std::string> > > parsedValues;
 
+  //regular expression used to match against flags later in the code
+  //kept here to avoid constant recompilation of the regex during the loop
+  const zypp::str::regex rxexpr("^--([^=]+)(=.*)?$");
+
   while ( true ) {
 
+    const int currOptInd = optind == 0 ?  1 : optind; //optind 0 always points to the command name
     int option_index = -1;      //index of the last found long option, same as in allOpts
-    int optc = getopt_long( argc, argv, shortopts.c_str(), longopts.data(), &option_index );
+    const int optc = getopt_long( argc, argv, shortopts.c_str(), longopts.data(), &option_index );
 
     if ( optc == -1 )
       break;
@@ -313,7 +337,7 @@ int parseCLI( const int argc, char * const *argv, const std::vector<CommandGroup
       case '?': {
         // wrong option in the last argument
         // last argument was a short option
-        if ( option_index == -1 && optopt)
+        if ( option_index == -1 && optopt > 0 )
           ZYPP_THROW( UnknownFlagException( std::string( 1, optopt ) ) );
         else
           ZYPP_THROW( UnknownFlagException( std::string( argv[optind - 1] ) ) );
@@ -326,6 +350,7 @@ int parseCLI( const int argc, char * const *argv, const std::vector<CommandGroup
       }
       default: {
         int index = -1;
+        bool longOpt = false;
         if ( option_index == -1 ) {
           //we have a short option
           auto it = shortOptIndex.find( (char) optc );
@@ -334,6 +359,7 @@ int parseCLI( const int argc, char * const *argv, const std::vector<CommandGroup
           }
         } else {
           //we have a long option
+          longOpt = true;
           index = option_index;
         }
 
@@ -349,6 +375,27 @@ int parseCLI( const int argc, char * const *argv, const std::vector<CommandGroup
 
           CommandOption &opt = allOpts[index];
 
+          // getopt_long supports unqiue abbreviation of flags, e.g. --reposd-dir can be abbreviated
+          // with --repos. We need to check here if the flag that was parsed is matching exactly
+          // the option name, otherwise we error out (bsc#1164543)
+          if ( longOpt ) {
+            const std::string scanned( argv[currOptInd] );
+
+            zypp::str::smatch what;
+            if ( zypp::str::regex_match( scanned, what, rxexpr ) && what.size() >= 2 ) {
+              if( opt.name != what[1] ) {
+                if ( onlyWarnOnAbbrevSwitches() ) {
+		  WAR << "Abbreviated commandline options will not be supported in future versions: --" << what[1] << ": use --" << opt.name << endl;
+		  Zypper::instance().out().warning( legacyCLIStr( what[1], opt.name, LegacyCLIMsgType::Abbreviated ) );
+		} else
+                  ZYPP_THROW( UnknownFlagException( scanned ) );
+              }
+            } else {
+              if ( !onlyWarnOnAbbrevSwitches() )
+                ZYPP_THROW( ZyppFlagsException( scanned + " did not match the expected flag format." ) );
+            }
+          }
+
           // check if a conflicting option was used before
           std::vector<std::string> conflictingList;
           for ( const auto &flagSet : conflictingFlags ) {
@@ -499,12 +546,7 @@ void renderHelp( const std::vector<CommandGroup> &options )
           std::cout << ">";
       }
 
-      std::cout << "\t" << opt.help;
-
-      auto defVal = opt.value.defaultValue();
-      if ( defVal )
-        std::cout << " Default: " << *defVal;
-
+      std::cout << "\t" << opt.optionHelp();
       std::cout << std::endl;
 
     }
@@ -524,7 +566,7 @@ std::string CommandOption::optionHelp() const
 {
   std::string optHelpTxt = help;
   auto defVal = value.defaultValue();
-  if ( defVal )
+  if ( defVal && defVal->size() )
     optHelpTxt.append(" ").append(str::Format(("Default: %1%")) %*defVal );
   return optHelpTxt;
 }
